; include file for BGLC commands for dynamic sceneries
; based on the (FS2000) Scenery_SDK.doc from Microsoft
; 12/23/2002 Edgar Knobloch


include fs6def.inc
include model_id.inc

D_LAT48 macro hi,lo
	dw	lo;
	dd	hi;
	endm

D_LON48 macro hi,lo
	dw 	lo		;
	dd 	hi		;
	endm

D_ALT48 macro hi,lo
	dw 	lo		;
	dd 	hi		;
	endm

CHANGE_PITCH macro goal,rate	; change pitch toward a goal at rate/second
	db 	01		; opcode (DYNOP_CHANGE_PITCH)
	dw	goal		; goal pitch, bank, or heading
	dw	rate 		; increment per second
	endm

CHANGE_BANK macro goal,rate	; change bank toward a goal at rate/second
	db 	02 		; opcode (DYNOP_CHANGE_BANK)
	dw 	goal 		; goal pitch, bank, or heading
	dw 	rate		; increment per second
	endm

CHANGE_HEADING macro goal, increment ; change heading toward a goal at rate/second
	db 	03		; opcode (DYNOP_CHANGE_HEADING)
	dw 	goal 		; goal pitch, bank, or heading
	dw 	increment 	; increment per second
	endm

SET_GEAR macro status 		; raise (0) or lower (1) the objects gear
	db 	04		; opcode (DYNOP_GEAR)
	db 	status	 	; gear up or down
	endm

SET_PITCH macro pitch		; set the objects pitch
	db 	05		; opcode (DYN_SET_PITCH)
	dw 	pitch 		; new pitch
	endm

SET_BANK macro bank		; set the objects bank
	db 	06 		; opcode (DYN_SET_BANK)
	dw 	bank 		; new bank
	endm

SET_HEADING macro heading	; set the objects heading
	db 	07 		; opcode (DYN_SET_HEADING)
	dw 	heading		; new heading
	endm

MOVE_X macro time, distance	; take N seconds to move object in X (+east, -west)
	db 	18 		; opcode (DYN_MOVE_DX) (012h)
	dw 	time		; over time in seconds
	dw 	distance 	; distance to move in Meters (+east, -west)
	endm

MOVE_Y macro time, distance	; take N seconds to move object in Y
	db 	19		; opcode (DYN_MOVE_DY) (013h)
	dw 	time		; over time in seconds
	dw 	distance 	; distance to move in Meters (+up, -down)
	endm

MOVE_Z macro time, distance	; take N seconds to move object in Z
	db 	20		; opcode (DYN_MOVE_DZ) (014h)
	dw 	time		; over time in seconds
	dw 	distance	; distance to move (+north, -south)
	endm

MOVE_XZ macro time, distance_x,distance_z ; take N seconds to move object in X and Z
	db 	21		; opcode (DYN_MOVE_DXZ) (015h)
	dw 	time		; over time in seconds
	dw 	distance_x	; distance to move in X-axis (+east, -west)
	dw 	distance_z	; distance to move in Z-axis (+north, -south)
	endm

MOVE_XY macro time, distance_x,distance_y 		; take N seconds to move object in X and Y
	db 	22		; opcode (DYN_MOVE_DXY) (016h)
	dw 	time		; over time in seconds
	dw 	distance_x	; distance to move in X-axis (+east, -west)
	dw 	distance_y	; distance to move in Y-axis (+up, -down)
	endm

MOVE_YZ macro time, distance_y,distance_z 		; take N seconds to move object in Y and Z
	db 	23		; opcode (DYN_MOVE_DYZ)
	dw 	time		; over time in seconds (017h)
	dw 	distance_y	; distance to move in Y-axis (+up, -down)
	dw 	distance_z	; distance to move in Z-axis (+north, -south)
	endm

MOVE_XYZ macro time, distance_x,distance_y,distance_z 	; take N seconds to move object in X, Y, and Z
	db 	24		; opcode (DYN_MOVE_DXYZ) (018h)
	dw 	time		; over time in seconds
	dw 	distance_x	; X distance to move (+east, -west)
	dw 	distance_y	; Y distance to move (+up, -down)
	dw 	distance_z	; Z distance to move (+north, -south)
	endm

MOVEA_X macro time,distance,factor 	; take N seconds to move with acceleration object in X
	db 	25		; opcode (DYN_MOVE_AX) (019h)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance	; distance to move (+east, -west)
	endm

MOVEA_Y macro time,distance,factor	; take N seconds to move with acceleration object in Y
	db 	26		; opcode (DYN_MOVE_AY) (01Ah)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance	; distance to move (+up, -down)
	endm

MOVEA_Z macro time,distance,factor	; take N seconds to move with acceleration object in Z (factor<4096 is slow down)
	db 	27		; opcode (DYN_MOVE_AZ) (01Bh)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance	; distance to move (+north, -south)
	endm

MOVEA_XZ macro time,distance_x,distance_z,factor	; take N seconds to move with acceleration object in X and Z
	db 	28		; opcode (DYN_MOVE_AXZ) (01Ch)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance_x	; distance to move in X-axis (+east, -west)
	dw 	distance_z	; distance to move in Z-axis (+north, -south)
	endm

MOVEA_XY macro time,distance_x,distance_y,factor 	; take N seconds to move with acceleration object in X and Y
	db 	29		; opcode (DYN_MOVE_AXY) (01Dh)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance_x	; distance to move in X-axis (+east, -west)
	dw 	distance_y	; distance to move in Y-axis (+up, -down)
	endm

MOVEA_YZ macro time,distance_y,distance_z,factor 	; take N seconds to move with acceleration object in Y and Z
	db 	30		; opcode (DYN_MOVE_AYZ) (01Eh)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance_y	; distance to move in Y-axis (+up, -down)
	dw 	distance_z	; distance to move in Z-axis (+north, -south)
	endm

MOVEA_XYZ macro time,distance_x,distance_y,distance_z,factor ; take N seconds to move with acceleration object in X, Y, and Z
	db 	31		; opcode 31 (DYN_MOVE_AXYZ) (01Fh)
	dw 	time		; over time in seconds
	dw 	factor		; acceleration factor
	dw 	distance_x	; X distance to move (+east, -west)
	dw 	distance_y	; Y distance to move (+up, -down)
	dw 	distance_z	; Z distance to move (+north, -south)
	endm

TURN macro time, goal, radius	; make a smooth circular turn
	db 	32		; opcode (DYN_TURN) (020h)
	dw 	time		; number of seconds to wait
	dw 	goal		; goal heading in pseudodegrees
	dw 	radius		; radius of the turn
	endm

DJUMP macro dest		; jump to a different command
				; relative destination
	local	start
start	label word
	db 	36		; opcode (DYN_JUMP) (024h)
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	endm

DCALL macro dest 		; call a subroutine (ONLY ONE LEVEL DEEP)
				; relative destination
	local	start
start	label word
	db 	37		; opcode (DYN_CALL) (025h)
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	endm

DRETURN macro 			; return from subroutine
	db 	38		; opcode (DYN_RET) (026h)
	endm

DWAIT macro time		; wait (pause) for N seconds
	db 	39		; opcode (DYN_WAIT) (027h)
	dw 	time		; number of seconds to wait
	endm

PAUSE macro 	 		; wait for another object to tell to continue
	db 	41		; opcode (DYN_PAUSE)
	endm

START macro dest		; start another object that is paused
				; object address
	local	start
start	label word
	db 	42		; opcode (DYN_START)
;	dw	dest-rel_base
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	endm

DIFE macro fail,index,value	; If token value is equal value, fall through to the next command; 
				; if not, jump to the fail destination
	local	start
start	label word
	db 	43		; opcode (DYN_IFE)
;	dw 	fail-rel_base	; fail relative destination
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	dw 	index		; index of the variable to compare
	dw 	value		; value to compare to
	endm

DIFNE macro fail,index,value	; If token value is not equal value, fall through to the next
				; command; if not, jump to the fail destination
	local	start
start	label word
	db 	44		; opcode (DYN_IFNE)
;	dw 	fail-rel_base	; fail relative destination
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	dw 	index		; index of the variable to compare
	dw 	value		; value to compare to
	endm

DIFL macro fail,index,value	; If token value is less than value, fall through to the next
				; command; if not, jump to the fail destination
	local	start
start	label word
	db 	45		; opcode (DYN_IFL)
;	dw 	fail-rel_base	; fail relative destination
	dw	(offset dest)-(offset start)
	ERRS16	(offset dest)-(offset start)
	dw 	index		; index of the variable to compare
	dw 	value		; value to compare to
	endm

DIFG macro fail,index,value	; If token value is greater than value, fall through to the next
				; command; if not, jump to the fail destination
	local	start
start	label word
	db 	46		; opcode (DYN_IFG)
;	dw 	fail-rel_base	; fail relative destination
	dw	(offset fail)-(offset start)
	ERRS16	(offset fail)-(offset start)
	dw 	index		; index of the variable to compare
	dw 	value		; value to compare to
	endm

SET_COLOR macro index,color	; set a color table value
	db 	49	; opcode (DYN_SET_COLOR)
	db 	index	; color index
	dw 	color	; new color
	endm

ADD_OBJECT macro model_id,start_offset,flag
			; manager command to activate an object
	local	start
start	label word
	db 	50	; opcode (DYN_ADD_OBJECT, 50) (032h)
	dw 	model_id	; id of the library model to use
				; relative offset to starting command
	dw	(offset start_offset)-(offset start)
	ERRS16	(offset start_offset)-(offset start)
	dw 	0	; internal index to the objects active record
;	{
;	FLAGS8 density:4; 	; flags (density to active at)
;	FLAGS8 air_traffic:1; 	; flags (air traffic enabled)
;	FLAGS8 ground_traffic:1; ; flags (ground traffic enabled)
;	FLAGS8 service:1; 	; flags (airport service objects such as fuel)
;	FLAGS8 other:1; 	; flags (other objects, cars, trains, boats)
;	}
	db	flag
	endm

SET_LLA macro lat_hi,lat_lo, lon_hi,lon_lo, alt_hi,alt_lo	
				; set the absolute position of an object
	db 	51		; opcode (DYN_SET_POSITION, 51) (033h)
	D_LAT48 lat_hi,lat_lo	; lat
	D_LON48 lon_hi,lon_lo	; lon
	D_ALT48 alt_hi,alt_lo	; alt
	endm

SET_LAT_ANCHOR macro lat_hi,lat_lo	; set the anchor latitude for longitude to meters conversions
	db 	52			; opcode (DYN_SET_LAT_ANCHOR, 52) (034h)
	D_LAT48 lat_hi,lat_lo		; lat anchor point
	endm

ADD_OBJECT_LIBRARY macro start_ptr_32,flag,guid1,guid2,guid3,guid4,bla
				; manager command to activate an object
	local	start
start	label dword
	db 	53		; opcode (DYN_ADD_OBJECT_32 ,53) (035h)
				; relative offset to starting command
	dd	(start_ptr_32)-(start)
;	ERRS16	(offset start_ptr_32)-(offset start)
	dw 	0		; internal index to the objects active record
;	{
;	FLAGS8 density:4; ; flags (density to active at)
;	FLAGS8 air_traffic:1; ; flags (air traffic enabled)
;	FLAGS8 ground_traffic:1; ; flags (ground traffic enabled)
;	FLAGS8 service:1; ; flags (airport service objects such as fuel)
;	FLAGS8 other:1; ; flags (other objects, cars, trains, boats)
;	}
	db	flag
	GUID128 <guid1,guid2,guid3,guid4>; library ID
	endm

MARKER_START macro	; starts the marker system
	db 56		; opcode (DYN_MARKER_START, 56)
	endm

MARKER_1 macro lat_hi,lat_lo,lon_hi,lon_lo,alt_hi,alt_lo,pbh ; one second marker
	db 	57		; opcode (DYN_LLA_MARKER_1 ,57)
	D_LAT48 lat_hi,lat_lo	; lat
	D_LON48 lon_hi,lon_lo	; lon
	D_ALT48 alt_hi,alt_lo	; alt
	PBH32 	pbh		; PBH
	endm

MARKER_5 macro lat_hi,lat_lo,lon_hi,lon_lo,alt_hi,alt_lo,pbh ; 5 second marker
	db 	58		; opcode (DYN_LLA_MARKER_5 ,58)
	D_LAT48 lat_hi,alt_lo	; lat
	D_LON48 lon_hi,lon_lo	; lon
	D_ALT48 alt_hi,alt_lo	; alt
	PBH32 	pbh		; PBH
	endm

MARKER_N macro time,lat_hi,lat_lo,lon_hi,lon_lo,alt_hi,alt_lo,pbh ; N-second marker
	db 	59		; opcode (DYN_LLA_MARKER_N ,59)
	db 	time		; time in seconds
	D_LAT48 lat_hi,alt_lo	; lat
	D_LON48 lon_hi,lon_lo	; lon
	D_ALT48 alt_hi,alt_lo	; alt
	PBH32 	pbh		; PBH
	endm

REFUEL_BOUNDS macro lat,lon,radius,dest,fuel_type 
				; bounds of a refueling box
	local	start
start	label word
	db 	61		; opcode (DYN_REFUEL_BOUNDS, 61)
	dd 	lat		; center lat
	dd 	lon		; center lon
	dd 	radius		; radius
	dw	(offset dest)-(offset start)	; relative destination
	ERRS16	(offset dest)-(offset start)
	dw 	fuel_type	; fuel type
	endm

REFUEL_PATH macro rate ; Go and refuel an aircraft in the refuel box
	db 	62	; opcode (DYN_REFUEL_PATH, 62)
	dd 	rate	; refuel rate in gallons/minute
	endm

LANDING_ABORT macro center_lat,center_lon,ground_alt,heading,box_length,box_width,vel,pattern_dir
				; Box to check in for other objects for a landing abort
	db 	64		; opcode (DYN_LANDING_ABORT, 64)
	dd 	center_lat	; latitude center of the ground area to check
	dd 	center_lon	; longitude center of the ground area to check
	dd 	ground_alt	; ground altitude and within 10 meters above
	db 	pattern_dir	; TRUE if right hand oval pattern
	dw 	vel		; objects current velocity
	dw 	heading		; heading angle of the box
	dd 	box_length	; length of the box (with the heading)
	dd 	box_width	; width of the box (across the heading)
	endm

DPARK macro lat_hi,lat_lo,lon_hi,lon_lo,alt_hi,alt_lo,heading	
				; move the object from the current location to the specified 
				; location at a speed of 3 Meters/second
	db 	65		; opcode (DYN_PARK, 65)
	D_LAT48 lat_hi,alt_lo	; lat
	D_LON48 lon_hi,lon_lo	; lon
	D_ALT48 alt_hi,alt_lo	; alt
	dd 	heading		; heading
	endm

ANTI_COLLISION_OFF macro status ; turns the object anti-collision system on/off
	db 	66	; opcode (DYN_COLLISION, 66)
	db 	status	; anti-collision status (TRUE = disabled)
	endm

IF_OBJECT_IN_BOX macro fail, center_lat, center_lon, center_alt, heading, box_length, box_width
				; asks if there is an object in the box
	local	start
start	label word
	db 	67		; opcode (DYN_IF_OBJECT_IN_BOX, 67)
	dw	(offset fail)-(offset start)	; relative fail destination if no object in the box
	ERRS16	(offset fail)-(offset start)
	dd 	center_lat	; center of the area
	dd 	center_lon
	dd 	center_alt	; ground alt
	dw 	heading		; heading angle of the box
	dd 	box_length	; length of the box (with the heading)
	dd 	box_width	; width of the box (across the heading)
	endm

FE_DYNAMIC_HEADER macro north_bounds_512,south_bounds_512,east_bounds,west_bounds,data_length
	local	start
	db 	4		; dynamic opcode (FEOP_DYNAMIC, 4)
	dw 	north_bounds_512  ; north bounds in 512 Meter Units
	dw 	south_bounds_512  ; south bounds in 512 Meter Units
	dw 	east_bounds	; east bounds in 16 pseudodegrees
	dw 	west_bounds	; west bounds in 16 pseudodegrees
	dw 	data_length-next_cmd	; length of the data NOT including header
next_cmd	label word
	endm

DYN_PATH_HEAD macro north, south, east, west
	dw 	north 	; north bounds of the dynamic file
	dw 	south 	; south bounds of the dynamic file
	dw 	east 	; east bounds of the dynamic file
	dw 	west 	; west bounds of the dynamic file
	dw 	12 dup(0)	;
	endm

DYN_EOF macro
	db	0ffh
	endm
